.section .iwram,"ax",%progbits
.align 2
.code 32

#define buffer  r1
#define dat  r3
#define pos  r4
#define inc  r5
#define vol  r6
#define samp  r7
#define tmp   r8
#define buf   r9

@ The trick here is to use single registers for multiple values.
@ Expensive operations like mul or str-in-ewram can be performed 
@ on multiple samples at once.

.macro  mixpair outreg
  ldrb   r11,[dat,pos,lsr#12]
  add    pos, pos, inc
  ldrb   r12,[dat,pos,lsr#12]
  add    pos, pos, inc

  @ sort out the signs 
  mov    r12, r12, lsl#24           @  0x80 00 00 00 
  @                                       signed          unsigned
  mov    r11, r11, lsl#24           @  0x80 00 00 00    0x7f 00 00 00
  mov    r11, r11, asr#8            @  0xf8 00 00 00    0x07 f0 00 00
  mov    r11, r11, lsr#16           @  0x00 00 0f 80    0x00 00 00 7f

  orr    r12, r11, r12, asr #8      @  r12 now holds 0xSS220S11  second and first bytes, sign byte/nibble
  mul    r11, r12, vol              @  r11 = vol*sample
  bic    r11, r11, #0x30000         @  clear the top bits for when we shift down.
  add    \outreg,  \outreg, r11, asr#2        @  4 channels, so divide by 4
.endm

@ void mix_loop(int, u16 *, sound_channel_t *) IN_IWRAM;
.global mix_loop
.type mix_loop,function
mix_loop:

  @ mix r0 samples from r2->data to r1
  ldr    r3,=lspace
  stmia  r3,{r4-r11,lr}
   
  ldr    dat,[r2, #data]
  ldr    pos,[r2, #position]
  ldr    inc,[r2, #increment]
  ldrb   vol,[r2, #volume]

  mov    r0,r0,lsr#2      @ 4 samples per loop

0:
  ldmia  buffer,{r7,r8}
  mixpair r7
  mixpair r8
  stmia  buffer!,{r7,r8}
  subs   r0,r0,#1
  bpl    0b
  @ end loop

  str    pos,[r2, #position]
  ldr    r3,=lspace
  ldmia  r3,{r4-r11,lr}
@ mov    pc,lr
  bx     lr


.global down_sample
.type down_sample,function
down_sample:

  @ down sample r0 samples from u16 buffer in r1 to u8 buffer in r2
  ldr    r3,=lspace
  stmia  r3,{r4-r11,lr}

  mov    r3,#0xff
  mov    r10,#0
  mov    r11,#0
  ldr    r9,=0x00630063

  mov    r0,r0,lsr#2          @ 4 samples per loop
                              @ doesn't matter if we accidentally mix more, 
                              @ as they are padded with 0s
1:
  @ a word 0x44332211 will be 44 33 22 11 in memory
  @ loading 2 words into memory will give:
  @ 00 AA 00 BB      00 CC 00 DD  ->  0x00AA00BB  0x00CC00DD
  @ and we want these in order AA,BB,CC,DD in u8*  0xAABBCCDD
  
  ldmia  r1,{r7,r8}           @ contains 4 lots of u16
  stmia  r1!,{r10,r11}        @ clear contents of tmp_buffer
  @ first the second byte - the lower bytes

  @ upper byte in r6    lower in r5
  and    r6,r3,r8,lsr#(16+6)  @ make sure 8 bit     r6 = (r8 >> 16)/64 & 0xff
  and    r5,r3,r8,lsr#6       @ lower byte in r5    r5 = (r8 /64)&0xff
  orr    r4,r5,r6,lsl#8       @ lower half in r4    r4 = r5|r6<<8

  @ then the first byte, which is really high
  
  @ upper byte in r6    lower in r5
  and    r6,r3,r7,lsr#(16+6)  @ make sure 8 bit     r6 = (r7 >> 16)/64 & 0xff
  and    r5,r3,r7,lsr#6       @ lower byte in r5    r5 = (r7 /64)&0xff
  orr    r5,r5,r6,lsl#8       @ upper half in r5    r5 = r5|r6<<8

  orr    r4,r5,r4,lsl#16      @ order matters      r4 = r5 | r4 << 16

  str    r4,[r2],#4           @ write 4 lots of u8 and increment
  
  subs   r0,r0,#1
  bpl    1b

  ldr    r3,=lspace
  ldmia  r3,{r4-r11,lr}
  bx     lr


.align 2

lspace: .space  24*4
.equ    data,           0
.equ    position,       4
.equ    increment,      8
.equ    volume,         20

.pool
.end
