#ifndef sxmp_private_h_seen
#define sxmp_private_h_seen

// not part of the public api, shouldn't really need to be included ouside the lib
#define USE_PROFILING 0

#include "gba.h"
#include "sxmp.h"

#ifndef IN_IWRAM
#define IN_IWRAM __attribute__ ((section (".iwram"),long_call))
#endif

#ifndef IN_EWRAM
#define IN_EWRAM __attribute__ ((section (".ewram")))
#endif


typedef struct
{
  s8 * mix_buffer;               // mix buffers for 4 channels each
  s8 * mix_buffer_2;
  s8 * current_mix_buffer;       // where we are currently writing to left
  s8 * current_mix_buffer_2;     // where we are currently writing to right
  u32  mix_buffer_length;        // size of each buffer (304)
  u16  samples_until_clock;      // depends on where we are in the frame vs tempo
  u16  samples_per_clock;        // samples used per beat (changes with BPM)
  u8   active;                   // which buffer is active, used for buffer swapping in vblank
} sound_var_t;

typedef struct
{
  const s8 * data; // points to sample data rom, or 0 for channel off
  u32 position;    // current position in the data (fixed point 20.12)
  u32 increment;   // step size (fixed point 20.12) for pitch
  u32 length;      // size of data (fixed point 20.12)
  u32 loop_length; // size of section to loop (fixed point 20.12) or 0 for no loop
  u8  volume;      // volume 0-64
  u8  channel;     // xm channel playing in this virtual channel
} sound_channel_t;

extern sound_var_t g_sound_vars;
extern sound_channel_t g_channel[CHANNEL_COUNT];

void sxmp_sound_mix(int samples_to_mix) IN_IWRAM;
void sxmp_no_channel_mix(int samples_to_mix) IN_IWRAM;
#endif
